#!/bin/tclsh

load tclrega.so

# - hier nur die IP Addresse anpassen
  set debug_mode 0
  set wait_between_status_requests 1
  set connection_timeout 500
  set wait_between_retries 200
  set status_retries 5
  set command_retries 10
  set matchport_host 192.168.0.129 
  set matchport_port 45000
  #set ws980command ""
  set ws980command "FFFF0B0006040419";  # aktuelle Daten holen


# - liest ein Byte vom Binärstring - liefert Zahl von 0 - 255
proc getb {idx} {
   global global_binout
   set sb [string index $global_binout $idx]
   scan $sb %c ascii
   return $ascii
}
# - liest ein Word vom Binärstring liefert Zahl von -32768 bis 32768
proc getw {idx} {
   global global_binout
   set sb [string index $global_binout $idx]
   scan $sb %c asciih
   set idx [expr $idx + 1]
   set sb [string index $global_binout $idx]  
   scan $sb %c asciil
   set zahl [expr $asciil + { $asciih * 256}]
   if {$zahl > 32767} then {set zahl ~[expr 65535 - $zahl]}
   return $zahl
} 
# - liest ein Longword vom Binärstring und liefert Zahl von 0 - 4294967295
proc getl {idx} {
   global global_binout
   set sb [string index $global_binout $idx]
   scan $sb %c ascii3
   set idx [expr $idx + 1]
   set sb [string index $global_binout $idx]  
   scan $sb %c ascii2
   set idx [expr $idx + 1]
   set sb [string index $global_binout $idx]  
   scan $sb %c ascii1   
   set idx [expr $idx + 1]
   set sb [string index $global_binout $idx]  
   scan $sb %c ascii0   
   set zahl [expr $ascii0 + {$ascii1 * 256} + {$ascii2 * 65536} + {$ascii3 * 16777216}]
   return $zahl
}

# - prüft checksumme des Binärstrings (Summe aller Bytes (2-80) AND FF - Byte 81 ist Checksumme) 
proc checksum {bin_str} {
   set bb 0
   set xx 0
   for {set i 2} {$i < 81} {incr i} {set xx [expr $xx + [getb $i]]}
   set xx [expr $xx & 255]
   if {$xx == [getb 81]} { set bb 1 }
   return $bb
}

# schreibt Logfile --> Mode = a für append, w für write (neue File) ----------------------------------------------------------------------
proc Write_Logfile {data mode} {
	set fileId [open "/etc/config/addons/weather/weather.log" $mode]
	fconfigure $fileId -buffering none
	puts $fileId $data
	close $fileId
}	

# öffnet Socket für TCP ------------------------------------------------------------------------------------------------------------------
proc ws980_opensocket {host port timeout} {
	global connected
	set channel [socket -async $host $port]
	set afterid [after $timeout {set connected "timeout"}]
	fileevent $channel writable "GetConnectError $channel $afterid"
	fileevent $channel readable "GetConnectError $channel $afterid"
	vwait connected
   if {[string compare $connected ""] == 0} { 
		return $channel
	} else {
		close $channel
		return -1
	}
}

# liest Binärdaten vom Socket ----------------------------------------------------------------------------------------------------------- 
proc ws980_Read {ws980_socket} {
	global global_ws980_timer_read
	global global_hexout
   global global_binout
	set hexout [read $ws980_socket 82];                     # Länge der gelesenen Daten 
	set global_binout $hexout;                              # Ausgabe in bin behalten wegen Auswertung
   binary scan $hexout H* hexout;                          # Binärwert nach Hexzahl
	set hexout [string toupper $hexout];                    # in Großschreibung ändern
	append global_hexout $hexout
	if {[string length $global_hexout] == 164} {set global_ws980_timer_read "read complete"}
}

# Fehlerbehandlung Socket ---------------------------------------------------------------------------------------------------------------
proc GetConnectError {channel afterid} {
	global connected
	set connected [fconfigure $channel -error]
	after cancel $afterid
	fileevent $channel readable ""
	fileevent $channel writable ""
}

   
# WS980 Steuerungsprozedur -------------------------------------------------------------------------------------------------------------
proc ws980_Command {debug_mode command maxwait_connect maxtries maxwait_between_tries matchport_host matchport_port} {
   
   if {$debug_mode == 1} {Write_Logfile "###########[clock format [clock sec] -format %H:%M:%S]: START COMMAND" "w"}
	if {$debug_mode == 2} {Write_Logfile "###########[clock format [clock sec] -format %H:%M:%S]: START COMMAND" "a"}

	#Initialisierung
	global global_hexout;                     # enthält die von WS980 empfangenen Daten 
	global global_binout;                     # dito aber binär
   global global_ws980_timer_read;

	set result_string "OK"
	set try_success "FALSE"
	set tries 0

	if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: Before Command" "a"}   
   
   set hexin $command ;   # Kommando setzen - für WS980
   
   if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: $command" "a"}
	if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: After Command" "a"}
	
	#Prüfen, ob bisher alles ok
	if {$result_string == "OK"} {
		#Übergebene Anzahl Versuche durchführen bis maximal definierte Versuchsanzahl erreicht oder letzter Versuch erfolgreich war
		for {set tries 0} {$tries < $maxtries && $try_success == "FALSE" } {incr tries} {
		
			#Result je Connect-Versuch initialisieren
			set result_string "OK"
			
			#Werte initialisieren
			set hexout ""
			set global_hexout ""
			set global_binout ""
         set global_ws980_timer_read ""

			#Binary-Command an ws980 senden und Rückgabewert empfangen und speichern
			if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: Before OpenSocket" "a"}
			set ws980_socket [ws980_opensocket $matchport_host $matchport_port $maxwait_connect]	
			if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: After OpenSocket: $ws980_socket" "a"}
			if {$ws980_socket != -1} {
				catch {
					fconfigure $ws980_socket -blocking false -translation binary -buffering none
					if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: After fconfigure" "a"}
					#ws980-Befehl von HEX in Binary umwandeln und an ws980 übertragen
					puts -nonewline $ws980_socket [binary format H* $hexin]
					if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: After WS980-Puts: $hexin" "a"}
					
               # Rückgabewert vom WS980 übertragen und von Binary nach HEX-String umwandeln
					if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: Before Read" "a"}
					after $maxwait_connect set global_ws980_timer_read "timeout"
					fileevent $ws980_socket readable "ws980_Read $ws980_socket"
					vwait global_ws980_timer_read
					fileevent $ws980_socket readable {}
					set hexout $global_hexout
					if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: After ws980-Read: $hexout / $global_ws980_timer_read" "a"}
				}
				close $ws980_socket
				if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: After CloseSocket" "a"}
			}

			#Prüfen, ob Socket-Kommunikation grundsätzlich erfolgreich war
			if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: Before Check (IN=$hexin / OUT=$hexout)" "a"}
			# Länge des Rückgabestrings (82 Byte empfangen = 164 chars)
         if {[string length $hexout] != 164} {
				set result_string "#[clock format [clock sec] -format %H:%M:%S]: Socket-Error /$command"
			} else {
            #Prüfen, ob Byte 1-4 der Rückgabe vom ws980 mit den Input-Bytes übereinstimmen
				if {[string range $hexout 0 7] == [string range $hexin 0 7]} {
					set final_result $hexout
				} else {
					set result_string "#[clock format [clock sec] -format %H:%M:%S]: Read-Error /$command"
					if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: Read-Error (IN=$hexin / OUT=$hexout)" "a"}
				}	
			}	
			if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: After Returnstring" "a"}
				
						
			#Wenn Kommunikation fehlerfrei war, dann Loop beenden, sonst laut festgelegtem Wert bis zum nächsten Versuch warten
			if {$result_string == "OK"} {
				set try_success "TRUE"
			} else {
				#x ms Warten
				after $maxwait_between_tries
			}				
			if {$debug_mode != 0} {Write_Logfile "[clock format [clock sec] -format %H:%M:%S]: After Wait-Between-Tries: $result_string" "a"}
		}
	}
   
   if {$result_string == "OK"} {   
      
      # Test ob das die aktuellen Wetterdaten sind (gibt noch Min. und Max. Werte etc.)
      if {[string range $final_result 0 9] == "FFFF0B0050"} {
         # hier das Parsen des Binärstrings
         
         set itemp [expr double([getw 7]) / 10]
         set atemp [expr double([getw 10]) / 10]
         set dewpt [expr double([getw 13]) / 10]
         set windc [expr double([getw 16]) / 10]
         set heati [expr double([getw 19]) / 10]
         set hum_i [getb 22]
         set hum_a [getb 24]
         set apres [expr double([getw 26]) / 10]
         set rpres [expr double([getw 29]) / 10]   
         set w_dir [getw 32]
         set w_spe [getw 35]   
         set w_gus [getw 38]  
         set rainh [expr double([getl 41]) / 10]
         set raind [expr double([getl 46]) / 10]
         set rainw [expr double([getl 51]) / 10]
         set rainm [expr double([getl 56]) / 10]
         set rainy [expr double([getl 61]) / 10]
         set raint [expr double([getl 66]) / 10]
         set l_lux [expr double([getl 71]) / 10000]
         set uvrad [expr double([getw 76]) / 10]
         set uvidx [getb 79]
         set check [checksum $global_binout]
         
         # Das habe ich mit switch nicht hinbekommen !!! geht kein kleiner mit Zahl ?!
         if [expr $w_dir <= 360]   {set windrichtung N}
         if [expr $w_dir <= 348]   {set windrichtung NNW}    
         if [expr $w_dir <= 326]   {set windrichtung NW}    
         if [expr $w_dir <= 303]   {set windrichtung NWW}    
         if [expr $w_dir <= 281]   {set windrichtung W}    
         if [expr $w_dir <= 258]   {set windrichtung SWW}    
         if [expr $w_dir <= 236]   {set windrichtung SW}
         if [expr $w_dir <= 213]   {set windrichtung SSW}
         if [expr $w_dir <= 191]   {set windrichtung S}
         if [expr $w_dir <= 168]   {set windrichtung SSO}
         if [expr $w_dir <= 146]   {set windrichtung SO}
         if [expr $w_dir <= 123]   {set windrichtung SOO}
         if [expr $w_dir <= 101]   {set windrichtung O}
         if [expr $w_dir <= 78]    {set windrichtung NOO}
         if [expr $w_dir <= 56]    {set windrichtung NO}
         if [expr $w_dir <= 33]    {set windrichtung NNO}
         if [expr $w_dir <= 11]    {set windrichtung N}
         append windrichtung " - $w_dir"
      
         rega_script "var x = dom.GetObject('WS__check');x.State('$check');"
         rega_script "var x = dom.GetObject('WS_Innentemperatur');x.State('$itemp');"
         rega_script "var x = dom.GetObject('WS_Aussentemperatur');x.State('$atemp');"
         rega_script "var x = dom.GetObject('WS_Taupunkt');x.State('$dewpt');"
         rega_script "var x = dom.GetObject('WS_Windchill');x.State('$windc');"
         rega_script	"var x = dom.GetObject('WS_Hitzeindex');x.State('$heati');" 
         rega_script "var x = dom.GetObject('WS_Luftfeuchte_Innen');x.State('$hum_i');"
         rega_script "var x = dom.GetObject('WS_Luftfeuchte_Aussen');x.State('$hum_a');"
         rega_script "var x = dom.GetObject('WS_Luftdruck_absolut');x.State('$apres');"
         rega_script "var x = dom.GetObject('WS_Luftdruck_relativ');x.State('$rpres');"
         rega_script "var x = dom.GetObject('WS_Windrichtung');x.State('$windrichtung');"
         rega_script "var x = dom.GetObject('WS_Windgeschwindigkeit');x.State('$w_spe');"
         rega_script "var x = dom.GetObject('WS_Windboe');x.State('$w_gus');"
         rega_script "var x = dom.GetObject('WS_Regen_Stunde');x.State('$rainh');"
         rega_script "var x = dom.GetObject('WS_Regen_Tag');x.State('$raind');"
         rega_script "var x = dom.GetObject('WS_Regen_Woche');x.State('$rainw');"
         rega_script "var x = dom.GetObject('WS_Regen_Monat');x.State('$rainm');"
         rega_script "var x = dom.GetObject('WS_Regen_Jahr');x.State('$rainy');"
         rega_script "var x = dom.GetObject('WS_Regen_Total');x.State('$raint');"
         rega_script "var x = dom.GetObject('WS_Sonne_Helligkeit');x.State('$l_lux');"
         rega_script "var x = dom.GetObject('WS_Sonne_UV');x.State('$uvrad');"
         rega_script "var x = dom.GetObject('WS_Sonne_UV_Index');x.State('$uvidx');"   
      }
   }
   
   if {$debug_mode != 0} {Write_Logfile "###########[clock format [clock sec] -format %H:%M:%S]: END COMMAND" "a"}
	if {$debug_mode != 0} {Write_Logfile " " "a"}
}  
   
# ---------------------------------------------------------------------------------------------------------------------------------------------------
# Main
# ---------------------------------------------------------------------------------------------------------------------------------------------------


  if {$debug_mode == 2} {Write_Logfile "###########[clock format [clock sec] -format %H:%M:%S]: START / ARG1=$wait_between_status_requests / ARG2=$connection_timeout / ARG3=$wait_between_retries / ARG4=$status_retries / ARG5=$command_retries / ARG6=$matchport" "w"}
  ws980_Command $debug_mode $ws980command $connection_timeout $command_retries $wait_between_retries $matchport_host $matchport_port
  if {$debug_mode == 2} {Write_Logfile "###########[clock format [clock sec] -format %H:%M:%S]: END" "a"}

   
#  FFFF0B0050040100D702008303FFF0040083050083063307240827A20928280A005E0B00010C00050E00000000100000000011000000121200000012130000001714000000171500065914160438170323A1
#  erster empfangsversuch OK!!!!!!!!!!!!!!!!!!!!!!!!!!!!!   
   



  
  
   
   
   
   
